<?php
/*
Plugin Name: Blogger Title Fix
Plugin URI: http://notions.okuda.ca/wordpress-plugins/blogger-title-fix/
Description: Replaces numerical titles (from Blogger import) with the first few words of the post.
Author: Poco
Version: 0.2
Author URI: http://notions.okuda.ca
*/


class ko_blogger_title_fix
{
	var $option_name = "ko_blogger_title_fix";

	function get_default_options()
	{
		$options = array(
			"excerpt_length" => 48,
			"cut_word" => false
		);
		
		return $options;
	}
	
	function get_options()
	{
		// If the options exists in the database then retrieve them
		if ( false !== get_option($this->option_name) )
		{
			// Now we can get the options out of the database.
			$options = get_option($this->option_name);
		}
		else
		{
			// Otherwise initialize them to the defaults.
			$options = $this->get_default_options();
		}
		
		return $options;
	}
	
	function save_options($options)
	{
		update_option($this->option_name, $options);
	}

	function fragment($str, $n, $delim='...', $word_cut=false)
	{
		$len = strlen($str);
		if ($len > $n)
		{
			if ($word_cut)
			{
				// This code would limit the output to about n characters and not cut any words.
				// This looks a bit better than the default, but this is not how Blogger reacts.
				preg_match('/(.{' . $n . '}.*?)\b/', $str, $matches);
				return rtrim($matches[1]) . $delim;
			}
			else
			{
				// Just take the first n characters, like Blogger does.
				return substr(trim($str), 0, $n) . $delim;
			}
		}
		else
		{
			return $str;
		}
	}

	function is_bad_title($title)
	{
		// Look for a title that is only decimal digits (or empty)
		return preg_match("/^\d*$/", $title);
	}

	function filter_title($content = '', $thispost = NULL)
	{
		global $post;
		$options = $this->get_options();

		// If we didn't get a post parameter then use the global one.
		if (!isset($thispost))
		{
			$thispost = post;
		}
		else
		{
			// In generate $thispost will only contain the ID and Title and we need more
			// So get it now.
			$thispost = get_post($thispost->ID);
		}

		if ($this->is_bad_title($content))
		{
			// Couldn't think of a better way to handle this.
			// Stole this code from get_the_excerpt except that it uses the available $thispost
			// instead of the global $post.
			$output = $thispost->post_excerpt;
			if ( !empty($thispost->post_password) ) { // if there's a password
				if ( $_COOKIE['wp-postpass_'.COOKIEHASH] != $thispost->post_password ) {  // and it doesn't match the cookie
					$output = "";  // empty title
				}
			}

			// If there is no excerpt then we take the content.  Again, there was no good function to call
			// here since they all assume $post is global.
			if ( '' == $output ) {
				$output = $thispost->post_content;
				$output = apply_filters('the_content', $output);
				$output = str_replace(']]>', ']]&gt;', $output);
				$output = strip_tags($output);
			}


			$newtitle = $this->fragment($output, $options["excerpt_length"], '...', $options["cut_word"]);
			return $newtitle;
		}
		else
		{
			return $content;
		}
	}

	function title_fix_page()
	{
		// Get the current value of the options.
		$options = $this->get_options();


		if (isset($_POST['reset_default']))
		{
			?><div class="updated"><p><strong><?php 
				// Store the default options to the database
				$options = $this->get_default_options();
				$this->save_options($options);
				_e('Settings Reset', 'Localization name');
			?></strong></p></div><?php
		}
		else if (isset($_POST['info_update']))
		{
			?><div class="updated"><p><strong><?php 

			if (isset($_POST['excerpt_length']))
			{
				$options["excerpt_length"] = $_POST['excerpt_length'];
			}
	
			if (isset($_POST['cut_word']))
			{
				$options["cut_word"] = true;
			}

			// Store the new options to the database
			$this->save_options($options);
			
			_e('Settings Saved', 'Localization name');
			
			?></strong></p></div><?php
		}

		?><div class=wrap>
			<form method="post">
				<h2>Blogger Title Fix</h2>
				<fieldset class="options" name="set1">
					<legend><?php _e('Excerpt Options', 'Localization name') ?></legend>
					Number of characters from the post to use in the title: <input type="text" name="excerpt_length" value="<?php echo $options["excerpt_length"] ?>" ><br />
					Cut title at word boundary: <input type="checkbox" name="cut_word" <?php if ($options["cut_word"]) { print ('checked="checked"'); } ?> >
				</fieldset>
				<div class="submit">
					<input type="submit" name="info_update" value="<?php
					_e('Update options', 'Localization name')
					?> " />
					<input type="submit" name="reset_default" value="<?php
					_e('Reset to Defaults', 'Localization name')
					?> " />
				</div>
			</form>
		</div><?php
	}
	
	
	function add_page()
	{
		if (function_exists('add_options_page'))
		{
			add_options_page('Blogger Title Fix', 'Blogger Title Fix', 10, basename(__FILE__), array(&$this, 'title_fix_page'));
		}
	}
}


$ko_blogger_title_fix_instance = new ko_blogger_title_fix();
add_filter('the_title', array(&$ko_blogger_title_fix_instance, 'filter_title'),10,2);
add_action('admin_menu', array(&$ko_blogger_title_fix_instance, 'add_page'));

?>
